import { ListPage } from "./list.po";
import { CalculatorPage } from "./calculator.po";
import { Navbar } from "./navbar.po";
import { browser, by, element } from "protractor";
import { PreferencesPage } from "./preferences.po";

describe("Calculate button - ", () => {
    let listPage: ListPage;
    let calcPage: CalculatorPage;
    let navBar: Navbar;
    let prefPage: PreferencesPage;

    beforeAll(async () => {
        prefPage = new PreferencesPage();
        listPage = new ListPage();
        calcPage = new CalculatorPage();
        navBar = new Navbar();
    });

    beforeEach(async () => {
        await prefPage.navigateTo();
        // disable evil option "empty fields on module creation"
        await prefPage.disableEvilEmptyFields();
        await browser.sleep(200);
    });

    it("check button status only depends on calculator (no link between calculators)", async () => {
        // start page
        await navBar.clickNewCalculatorButton();
        await browser.sleep(200);

        // open PAB: chute calculator
        await listPage.clickMenuEntryForCalcType(12);
        await browser.sleep(200);

        // start page
        await navBar.clickNewCalculatorButton();
        await browser.sleep(200);

        // open PAB: dimensions
        await listPage.clickMenuEntryForCalcType(5);
        await browser.sleep(200);

        // fill width field with invalid data
        const inputW = calcPage.getInputById("W");
        await inputW.clear();
        await browser.sleep(20);
        await inputW.sendKeys("-1");
        await browser.sleep(200);

        // check that "compute" button is inactive
        calcPage.checkCalcButtonEnabled(false);

        // back to PAB: chute
        await navBar.clickCalculatorTab(0);
        await browser.sleep(200);

        // check that "compute" button is active
        calcPage.checkCalcButtonEnabled(true);

        // back to PAB: dimensions
        await navBar.clickCalculatorTab(1);
        await browser.sleep(200);

        // check that "compute" button is inactive
        calcPage.checkCalcButtonEnabled(false);
    });

    describe("check button status in prébarrages - ", () => {
        it("invalid data in Q input", async () => {
            // start page
            await navBar.clickNewCalculatorButton();
            await browser.sleep(200);

            // open prébarrages calculator
            await listPage.clickMenuEntryForCalcType(30);
            await browser.sleep(200);

            // Q input
            const inputQ = element(by.id("Q"));
            await inputQ.clear();
            await browser.sleep(200);
            await inputQ.sendKeys("-1");
            await browser.sleep(200);

            calcPage.checkCalcButtonEnabled(false);

            // upstream item
            // look for g element with id starting by "flowchart-amont-"
            // Mermaid changed the way ids are generated, it preprends "flowchart-" and appends a number to the id given in the graph description
            const upstream = element(by.css("g[id^='flowchart-amont-']"));
            // should be displayed in error
            expect(await upstream.getAttribute('class')).toContain("node-highlighted-error"); // upstream item is now selected by default (was 'node-error')
        });

        it("add basin, invalid data in Q input", async () => {
            // start page
            await navBar.clickNewCalculatorButton();
            await browser.sleep(200);

            // open prébarrages calculator
            await listPage.clickMenuEntryForCalcType(30);
            await browser.sleep(200);

            // "add basin" button
            const addBasinBtn = element(by.id("add-basin"));
            await addBasinBtn.click();
            await browser.sleep(200);

            // upstream item
            const upstream = element(by.css("g[id^='flowchart-amont-']")); // Mermaid generated id
            await upstream.click();
            await browser.sleep(200);

            // invalid data in Q input
            const inputQ = element(by.id("Q"));
            await inputQ.clear();
            await browser.sleep(200);
            await inputQ.sendKeys("-1");
            await browser.sleep(200);

            // calculate button disabled ?
            calcPage.checkCalcButtonEnabled(false);

            // upstream item displayed in error ?
            expect(await upstream.getAttribute('class')).toContain("node-highlighted-error"); // upstream item is now selected by default (was 'node-error')

            // valid data in Q input
            await inputQ.clear();
            await browser.sleep(200);
            await inputQ.sendKeys("1");
            await browser.sleep(200);

            // calculate button still disabled ? (the basin is not connected to anything)
            calcPage.checkCalcButtonEnabled(false);

            // upstream item displayed not in error ?
            expect(await upstream.getAttribute('class')).not.toContain("node-error");
        });
    });
});
