import { AppPage } from "./app.po";
import { ListPage } from "./list.po";
import { CalculatorPage } from "./calculator.po";
import { Navbar } from "./navbar.po";
import { SideNav } from "./sidenav.po";
import { browser, by, element } from "protractor";
import { PreferencesPage } from "./preferences.po";

const fs = require("fs");
const path = require("path");
const os = require("os");

let startPage: AppPage;
let listPage: ListPage;
let calcPage: CalculatorPage;
let navbar: Navbar;
let sidenav: SideNav;
let prefPage: PreferencesPage;

async function saveSession(): Promise<string> {
    await calcPage.clickSaveCalcButton();
    await browser.sleep(500);

    // see: https://stackoverflow.com/questions/21935696/protractor-e2e-test-case-for-downloading-pdf-file
    const filename = path.resolve(os.homedir(), "Téléchargements/session.json");
    if (fs.existsSync(filename)) {
        // Make sure the browser doesn't have to rename the download.
        fs.unlinkSync(filename);
    }

    // Le code laissé en commentaire tente de corriger un bug :
    // il s'écoule 40 secondes entre le clic sur le bouton menu (en haut à gauche) et l'ouverture du sidenav.
    // Ceci ne se produit que lorsqu'on sauve effectivement la session : si on annule la sauvegarde, il n'y a pas de délai.
    // https://stackoverflow.com/questions/75235558/delay-after-downloading-a-file-in-protractor-test

    //browser.manage().timeouts().implicitlyWait(100);
    //browser.ignoreSynchronization = true;
    // await browser.waitForAngularEnabled(false);

    if (true) {
        await calcPage.getSaveSessionButton().click();
    } else {
        const cancel = element(by.css("dialog-save-session button.mat-primary"));
        await cancel.click();
    }
    await browser.sleep(200);
    // browser.ignoreSynchronization = false;
    // await browser.waitForAngularEnabled(true);

    // browser.executeScript('window.stop();');

    // const wins = await browser.driver.getAllWindowHandles();
    // await browser.switchTo().window(wins[0]);

    // await browser.switchTo().activeElement();

    // const bd = element(by.css("body"));
    // await browser.actions().mouseMove(bd, { x: 0, y: 0 }).click().perform();

    // await navbar.clickCalculatorTab(0);
    // await browser.sleep(200);

    // browser.actions().sendKeys(protractor.Key.ESCAPE).perform();

    return filename;
}

async function loadSession(path: string) {
    await navbar.clickMenuButton();
    await browser.sleep(200);

    await sidenav.clickLoadSessionButton();
    await browser.sleep(200);

    await sidenav.loadSessionFile(path);
}

function expectNumber(msg: string, val: number, expected: number) {
    if (val !== expected) {
        console.log(msg, "got", val, "expected", expected);
    }
    expect(val).toEqual(expected);
}

/**
 * Save and load (serialise and unserialise) calculators to/from JSON files
 */
describe("ngHyd − save and load sessions", () => {
    beforeAll(() => {
        startPage = new AppPage();
        listPage = new ListPage();
        calcPage = new CalculatorPage();
        navbar = new Navbar();
        sidenav = new SideNav();
        prefPage = new PreferencesPage();
    });

    beforeEach(() => {
        jasmine.DEFAULT_TIMEOUT_INTERVAL = 45 * 60 * 1000; // 45 min
        browser.manage().window().setPosition(2000, 30);
    });

    it("when loading session-6-calc.test.json file from home page, 6 calculators should be loaded", async () => {
        await startPage.navigateTo();

        await loadSession("./session/session-6-calc.test.json");
        await browser.sleep(1000);

        expect(await navbar.getAllCalculatorTabs().count()).toBe(6);
    });

    it("when loading session-optional-params.test.json file from home page, the calculator should be loaded", async () => {
        await startPage.navigateTo();

        await loadSession("./session/session-optional-params.test.json");
        await browser.sleep(200);

        expect(await navbar.getAllCalculatorTabs().count()).toBe(1);
    });

    it("when saving a calculator, the current parameter values should be found in the file", async () => {
        // disable evil option "empty fields on module creation"
        await prefPage.navigateTo();
        await prefPage.disableEvilEmptyFields();
        await browser.sleep(200);
        await navbar.clickNewCalculatorButton();

        await listPage.clickMenuEntryForCalcType(2); // Section paramétrée
        await browser.sleep(500);

        await calcPage.changeSelectValue(calcPage.getSelectById("select_section"), 2); // mode "trapezoidal"

        await calcPage.getInputById("Ks").clear(); // coefficient de Strickler
        await browser.sleep(1000);
        await calcPage.getInputById("Ks").sendKeys("42");
        await browser.sleep(1000);

        const filename = await saveSession();
        await browser.sleep(1000);
        const fileContent = fs.readFileSync(filename, { encoding: "utf8" });

        expect(fileContent).toContain(`"nodeType":"SectionTrapeze"`);
        expect(fileContent).toContain(`{"symbol":"Ks","mode":"SINGLE","value":42}`);
    });

    it("select value must be recovered when loading a session file", async () => {
        // start page
        await startPage.navigateTo();
        await browser.sleep(200);

        const calcTypes = await listPage.getAvailableCalcTypes();

        const excludedCalculators = [
            34, // vérificateur (nécessite d'ouvrir plusieurs calculettes)
        ];

        for (let i = 0; i < calcTypes.length; i++) {
            const ct = calcTypes[i];
            if (!excludedCalculators.includes(ct)) {
                if (i == 0) {
                    // enable evil option "empty fields on module creation"
                    await prefPage.navigateTo();
                    await prefPage.disableEvilEmptyFields();
                    await browser.sleep(200);

                    // start page
                    await navbar.clickNewCalculatorButton();
                    await browser.sleep(200);
                }
                else {
                    // empty session
                    await navbar.clickMenuButton();
                    await browser.sleep(200);
                    await sidenav.clickNewSessionButton();
                    await browser.sleep(200);
                }
                console.log("calc", ct);

                // open calculator
                await listPage.clickMenuEntryForCalcType(ct);
                await browser.sleep(200);

                // detect selects
                const selects = calcPage.getAllCalculatorSelects();
                const nsel = await selects.count();
                for (let s = 0; s < nsel; s++) {  // /!\ ElementArrayFinder.each() is ASYNCHRONOUS !! https://www.protractortest.org/#/api?view=ElementArrayFinder.prototype.each
                    const sel = selects.get(s);
                    const selId = await sel.getAttribute("id");

                    const options = await calcPage.getMatselectOptionsText(sel);
                    const optionCount = options.length;

                    if (optionCount > 0) {
                        // index of current selected option
                        const optTxt = await calcPage.getMatselectCurrentOptionText(sel);
                        const ind = options.indexOf(optTxt);

                        // select next select option (optionally looping)
                        const nextInd = (ind + 1) % optionCount;
                        await calcPage.changeSelectValue(sel, nextInd);
                        await browser.sleep(200);

                        // save session
                        const filename = await saveSession();
                        await browser.sleep(200);

                        // load session
                        await loadSession(filename); // bug here : the click on the menu button (top left) takes 40s to take effect and open the side nav!)
                        await browser.sleep(200);
                        // the displayed calculator is now the loaded one

                        // check the calculator has been loaded
                        // expect(await navbar.getCalculatorEntriesCount()).toEqual(2);
                        expectNumber("num calcs", await navbar.getCalculatorEntriesCount(), 2);

                        // check the select in the loaded session points to the same option
                        const sel2 = calcPage.getSelectById(selId);

                        // check the same option is in the select
                        const optTxt2 = await calcPage.getMatselectCurrentOptionText(sel2);
                        const ind2 = options.indexOf(optTxt2);
                        //expect(ind2).toEqual(nextInd);
                        expectNumber("opt index", ind2, nextInd);

                        // close last calculator (the loaded one)
                        await navbar.middleClickCalculatorTab(1);
                        await browser.sleep(200);

                        // check last calculator has been closed
                        // expect(await navbar.getCalculatorEntriesCount()).toEqual(1);
                        expectNumber("num calcs(2)", await navbar.getCalculatorEntriesCount(), 1);
                    }
                }
            }
        }
    });
});
