import { browser, by, element, ElementFinder } from "protractor";

export class PreferencesPage {
    navigateTo() {
        return browser.get("/#/setup");
    }

    getHeader1() {
        return element(by.css("h1"));
    }

    getLanguageSelect() {
        return element(by.css(`mat-select[data-testid="language-select"]`));
    }

    getEmptyFieldsCheckbox() {
        return element(by.css(`mat-checkbox#cb_emptyFields`));
    }

    getInputLabels() {
        return element.all(by.css("label.mat-form-field-label"));
    }

    getNumericFormFields() {
        return element.all(by.css(`mat-form-field[data-testclass="numeric-input"]`));
    }

    getInputForField(ff: ElementFinder) {
        return ff.element(by.css("input"));
    }

    /**
     * find input with given "name" attribute
     */
    getInputFromName(name: string) {
        const cssSelector: string = `input[name="${name}"]`;
        return element(by.css(cssSelector));
    }

    getErrorsForField(ff: ElementFinder) {
        return ff.element(by.css("mat-error"));
    }

    async changeLanguage(index: number) {
        const select = this.getLanguageSelect();
        await select.click();
        const optionId = ".cdk-overlay-container mat-option#mat-option-" + index;
        const option = element(by.css(optionId));
        await option.click();
    }

    async enableEvilEmptyFields() {
        const cb = this.getEmptyFieldsCheckbox();
        const underlyingCB = cb.element(by.css(`input.mat-checkbox-input`));
        if (!underlyingCB.isSelected()) {
            await cb.click();
        }
    }

    async disableEvilEmptyFields() {
        const cb = this.getEmptyFieldsCheckbox();
        const underlyingCB = cb.element(by.css(`input.mat-checkbox-input`));
        if (underlyingCB.isSelected()) {
            await cb.click();
        }
    }

    async setIterationCount(n: number) {
        const input = this.getInputFromName("nmi");
        input.clear();
        await input.sendKeys(n.toString());
    }
}
